import bpy
import os

# CONST #

# TODO check if materials are correct
MATERIAL_PREFIXES = {
    "road": "1ROAD",
    "curb": "1KERB",
    "grass": "1GRASS",
    "sand": "1SAND",
    "wall": "1WALL",
    "pit": "1PIT",
    "out": "1OUT"
}

# FUNCTIONS

def rename_objects(self, objects: bpy.types.Object, prefix: str):
    if prefix not in MATERIAL_PREFIXES.values():
        self.report({'ERROR'}, f"Invalid prefix {prefix}. Aborting rename.")
        return False
    
    for obj in objects:
        if obj.name.startswith(tuple(MATERIAL_PREFIXES.values())):
            self.report({'ERROR'}, f"Object already is an AC material! Reset before reassigning.")
            continue
        obj.name = prefix + '.' + obj.name

    return True


def remove_prefix(self, objects):
    for obj in objects:
        for prefix in MATERIAL_PREFIXES.values():
            # replace AC-Prefix
            if obj.name.startswith(prefix):
                obj.name = obj.name.replace(prefix, '')
                # replace delimiter
                if obj.name.startswith(('.', '_', '-')):
                    obj.name = obj.name[1:]

                self.report({'INFO'}, f"Removed AC-Material from object {obj.name}")
                return True
            
    return False


# OPERATOR #

class RemoveMaterial(bpy.types.Operator):
    bl_idname = "object.remove_material"
    bl_label = "Remove Material"
    bl_description = "Removing AC-Material prefix from object name"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        # TODO implement grip level for remove
        remove_prefix(self, context.selected_objects)

        return {'FINISHED'}
    

class MakeRoad(bpy.types.Operator):
    bl_idname = "object.make_road"
    bl_label = "Make Road"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        # TODO implement grip level
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['road'])

        return {'FINISHED'}
    

class MakeGrass(bpy.types.Operator):
    bl_idname = "object.make_grass"
    bl_label = "Make Grass"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['grass'])

        return {'FINISHED'}
    

class MakeCurb(bpy.types.Operator):
    bl_idname = "object.make_curb"
    bl_label = "Make Curb"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['curb'])

        return {'FINISHED'}
    
class MakeWall(bpy.types.Operator):
    bl_idname = "object.make_wall"
    bl_label = "Make Wall"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['wall'])

        return {'FINISHED'}
   

class MakeSand(bpy.types.Operator):
    bl_idname = "object.make_sand"
    bl_label = "Make Sand"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['sand'])

        return {'FINISHED'}
    
class MakePit(bpy.types.Operator):
    bl_idname = "object.make_pit"
    bl_label = "Make Pit"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['pit'])

        return {'FINISHED'}
            
class MakeOut(bpy.types.Operator):
    bl_idname = "object.make_out"
    bl_label = "Make Out"
    bl_description = "Renaming object so AC-material will be applied"

    @classmethod
    def poll(cls, context):
        objects = context.selected_objects
        return objects is not None

    def execute(self, context):
        rename_objects(self, context.selected_objects, MATERIAL_PREFIXES['out'])

        return {'FINISHED'}
        