import bpy
import os

from .Properties import AC_FOLDER_NAME
from .Functions import TrackFolder, find_ac_installation, get_blender_path, get_properties


class UI_Tools(bpy.types.Panel):
    """Creates a Panel in the scene context of sidebar"""
    bl_label = "Tools"
    bl_idname = "SCENE_PT_ac_tools_tools"
    bl_space_type = 'VIEW_3D'  # Correct space type for the 3D View
    bl_region_type = 'UI'  # 'UI' is the region type for the Sidebar
    bl_context = "objectmode"  # You can change this depending on the context where you want the panel to appear
    bl_category = "AC Tools"

    def draw(self, context):
        layout = self.layout


class UI_Objects(bpy.types.Panel):
    """Creates a Panel in the scene context of sidebar"""
    bl_label = "AC Objects"
    bl_idname = "SCENE_PT_ac_tools_objects"
    bl_space_type = 'VIEW_3D'  # Correct space type for the 3D View
    bl_region_type = 'UI'  # 'UI' is the region type for the Sidebar
    bl_context = "objectmode"  # You can change this depending on the context where you want the panel to appear
    bl_category = "AC Tools"
    bl_parent_id = "SCENE_PT_ac_tools_tools"

    def draw(self, context):
        layout = self.layout

        props = get_properties(context)

        layout.prop(props, "grid_align", text="Align")
        layout.prop(props, "grid_offset", text="Offset")
        #layout.prop(props, "track_align", text="Align Track") # make function for align first...
        box = layout.box()
        box.operator("object.create_start", icon='RESTRICT_SELECT_OFF')
        box.operator("object.create_hotlap_start", icon='RESTRICT_SELECT_OFF')
        box.operator("object.create_pit", icon='PMARKER_ACT')
        box.operator("object.create_timing", icon='TIME')




class UI_Materials(bpy.types.Panel):
    """Creates a Panel in the scene context of sidebar"""
    bl_label = "AC Materials"
    bl_idname = "SCENE_PT_ac_tools_materials"
    bl_space_type = 'VIEW_3D'  # Correct space type for the 3D View
    bl_region_type = 'UI'  # 'UI' is the region type for the Sidebar
    bl_context = "objectmode"  # You can change this depending on the context where you want the panel to appear
    bl_category = "AC Tools"
    bl_parent_id = "SCENE_PT_ac_tools_tools"

    def draw(self, context):
        layout = self.layout

        box = layout.box()
    
        col = box.column()
        col.operator("object.remove_material", icon='TRASH')
        col.separator()
        col.label(text='Surfaces:')
        col.operator("object.make_road", icon='VIEW_PERSPECTIVE', text='Road')
        col.operator("object.make_curb", icon='VIEW_PERSPECTIVE', text='Curb')
        col.operator("object.make_grass", icon='VIEW_PERSPECTIVE', text='Grass')
        col.operator("object.make_sand", icon='VIEW_PERSPECTIVE', text='Sand')
        col.operator("object.make_pit", icon='VIEW_PERSPECTIVE', text='Pit')
        col.operator('object.make_out', icon='VIEW_PERSPECTIVE', text='Asphalt (No Track)')
        col.label(text='Collision:')
        col.operator("object.make_wall", icon='MESH_CUBE', text='Wall')

        row = layout.row()
        row.label(text="""\nTo modify or create materials check your './ac_track/data/surfaces.ini' and adjust it.\
                  \nThe names of the materials have to math the object prefixes! \
                  """, icon='INFO')


class UI_ProjectSetup(bpy.types.Panel):
    """Creates a Panel in the scene context of sidebar"""
    bl_label = "Project Setup"
    bl_idname = "SCENE_PT_acetrack_project_setup"
    bl_space_type = 'VIEW_3D'  # Correct space type for the 3D View
    bl_region_type = 'UI'  # 'UI' is the region type for the Sidebar
    bl_context = "objectmode"  # You can change this depending on the context where you want the panel to appear
    bl_category = "AC Tools"

    def draw(self, context):
        props = get_properties(context)
        layout = self.layout

        scene = context.scene

        if not bpy.data.filepath:
            layout.label(text='Save .blend first')
            return

        # TRACK FOLDER
        tf = TrackFolder(props.track_folder)
        if not tf.exists():
            box = layout.box()
            box.operator("project.init", text="Create Default Track Folder")
            layout.label(text="...or choose folder...")
            layout.prop(props, "track_folder", text="")
        else:
            box = layout.box()
            box.operator("project.edit_track_ui", icon='CURRENT_FILE')
            box.operator("project.edit_surface_ini", icon='CURRENT_FILE')
            box.label(text="Track Folder:")
            box.prop(props, "track_folder", text="")
            box.operator("project.open_track_folder", icon='FILEBROWSER')
            box.operator("project.rename_track", text="Rename Track", icon='GREASEPENCIL')
            #box.operator("project.edit_surface_ini")


class UI_ProjectExport(bpy.types.Panel):
    """Creates a Panel in the scene context of sidebar"""
    bl_label = "Project Export"
    bl_idname = "SCENE_PT_acetrack_project_export"
    bl_space_type = 'VIEW_3D'  # Correct space type for the 3D View
    bl_region_type = 'UI'  # 'UI' is the region type for the Sidebar
    bl_context = "objectmode"  # You can change this depending on the context where you want the panel to appear
    bl_category = "AC Tools"

    def draw(self, context):
        props = get_properties(context)
        layout = self.layout

        if not bpy.data.filepath:
            layout.label(text='Save .blend first')
            return

        # AC INSTALLATION
        box = layout.box()
        box.label(text="AC Folder:")
        box.operator("project.ks_finder", text="Find AC Installation", icon='ZOOM_ALL')
        box.prop(props, "ac_installation", text="")

        # EXPORT FBX
        box = layout.box()
        box.label(text="Scene to FBX")
        box.operator("project.export_fbx_for_ac", text="Export FBX for KsEditor", icon='EXPORT')

        box.operator("project.ks_editor", text="KsEditor", icon='AUTO')
        box.label(text="Change defaults under 'Utilities > Data Editor'", icon='INFO')

        # INSTALL TO AC
        box = layout.box()
        box.label(text="Track Folder to AC")
        box.prop(props, "disable_export_checks", text="Disable Export Checks")
        if not props.disable_export_checks and track_is_updated(props, box):
            box.label(text='Not synced!', icon='WARNING_LARGE')
        box.operator("project.export_track_folder", icon='VIEW_PERSPECTIVE', text="Install Track to AC")
    

# TODO optimize performance (only compare when reasonable) # e.g. with variable (detect once -> set, reset when install executed) <- brings other risks/problems
from filecmp import dircmp
def track_is_updated(props, layout: bpy.types.UILayout):
    track_folder = get_blender_path(props.track_folder)
    if not track_folder or not os.path.exists(track_folder):
        return False # not there yet...

    ac_track_folder = os.path.join(props.ac_installation, 'content', 'tracks', os.path.basename(os.path.dirname(track_folder)))
    if not ac_track_folder or not os.path.exists(ac_track_folder):
        print("Track-Folder not exported to AC yet.")
        return True # if its not there it must be new
    
    compare_path = ac_track_folder
    diff = []
    for path, dirs, files in os.walk(track_folder):
        if path != track_folder:
            compare_path = os.path.join(ac_track_folder, os.path.basename(path))
        comparison = dircmp(path, compare_path)
        if not comparison.diff_files and not comparison.funny_files and comparison.left_list == comparison.right_list:
            #print(f"{path} seems okay to me...")
            pass
        else:
            #print(f"Comparison '{path}':\n> Diff: {comparison.diff_files}\n> Funny: {comparison.funny_files}\n> Same: {comparison.same_files}\nMod Folder: {comparison.left_list}\nAC Folder: {comparison.right_list}")
            diff.extend(comparison.diff_files)
            diff.extend(comparison.funny_files)
            diff.extend(comparison.left_only) # new files
            # diff.extend(comparison.right_only) # dont check files added by the game

    if diff:
        box = layout.box()
        for file in diff:
            row = box.row()
            row.label(text=file, icon='FILE_NEW')
        return True
    return False